from filum_utils.clients.notification.client import NotificationClient
from filum_utils.clients.notification.enums import (
    NOTIFICATION_ERROR_MESSAGE_MAPPINGS,
    ErrorType,
    NotificationErrorMessage,
    PublisherType,
    RoutePath,
)
from filum_utils.clients.notification.types import Route

__all__ = [
    Route,
    PublisherType,
    RoutePath,
    ErrorType,
    NotificationErrorMessage,
    NOTIFICATION_ERROR_MESSAGE_MAPPINGS,
    NotificationClient,
]



import json
from datetime import datetime
from typing import Any, Dict

from dateutil.tz import UTC

from filum_utils.clients.common import BaseClient
from filum_utils.clients.notification.enums import (
    ToolLogActionEnum,
    ToolLogCodeEnum,
    ToolLogTypeEnum,
)
from filum_utils.clients.notification.types import Route
from filum_utils.config import config
from filum_utils.enums import ObjectType


class NotificationClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.NOTIFICATION_BASE_URL,
            username=config.NOTIFICATION_USERNAME,
            password=config.NOTIFICATION_PASSWORD,
        )

    def create_notification(
        self,
        publisher_type: str,
        title: str,
        subtitle: str,
        route: Route,
        member_account_id: int = None,
        member_organization_id: str = None,
    ):
        notification = {
            "title": title,
            "subtitle": subtitle,
            "publisher_type": publisher_type,
            "route": route,
            "member_account_id": member_account_id,
            "member_organization_id": member_organization_id,
        }

        return self._request(
            method="POST", endpoint="/internal/notifications", data=notification
        )

    def log_tool_execution(
        self,
        title: str,
        subtitle: str,
        tool_id: str,
        input_data: Dict[str, Any],
        output_data: Dict[str, Any],
        execution_time: int,
        status: str,
        organization_id: str,
        account_id: int = None,
    ):
        log_data = {
            "code": ToolLogCodeEnum.EXECUTE,
            "type": ToolLogTypeEnum.INFO,
            "data": json.dumps(
                {
                    "toolId": tool_id,
                    "input": input_data,
                    "output": output_data,
                    "executionTime": execution_time,
                    "status": status,
                }
            ),
            "title": title,
            "subtitle": subtitle,
            "object_type": ObjectType.TOOL,
            "object_id": tool_id,
            "member_organization_id": organization_id,
            "action": ToolLogActionEnum.EXECUTE,
            "created": datetime.now().astimezone(tz=UTC).isoformat(),
        }

        if account_id:
            log_data["member_account_id"] = account_id

        return self._request(
            method="POST",
            endpoint="/internal/logs",
            data=log_data,
        )



import re
from typing import Any, Dict, Optional

from filum_utils.clients.filum import FilumClient
from filum_utils.utils.automated_action.enums import (
    TRANSLATION_FORMAT_REGEX,
    ConversationProperty,
    FilumEventName,
    FilumTrackEventProperty,
    SurveyResponseProperty,
)


def generate_dict(
    source: Dict[str, Any],
    property_mappings: Dict[str, str],
) -> Dict[str, Any]:
    result = {}
    for source_key, target_key in property_mappings.items():
        result[target_key] = source.get(source_key)
    return result


def get_survey_response(event_data: Dict[str, Any]) -> Dict[str, Any]:
    if (
        event_data.get(FilumTrackEventProperty.EVENT_NAME)
        != FilumEventName.SURVEY_ANSWERED
    ):
        return {}
    return {
        SurveyResponseProperty.ID: event_data.get(
            FilumTrackEventProperty.SURVEY_RESPONSE_ID
        )
    }


def get_conversation(event_data: Dict[str, Any]) -> Dict[str, Any]:
    if (
        event_data.get(FilumTrackEventProperty.EVENT_NAME)
        not in FilumEventName.get_conversation_event_names()
    ):
        return {}
    return {
        ConversationProperty.ID: event_data.get(FilumTrackEventProperty.CONVERSATION_ID)
    }


def get_follow_up_text_translation(
    event_data: Dict[str, Any],
    filum_client: FilumClient,
) -> Optional[Dict[str, Any]]:
    if (
        event_data.get(FilumTrackEventProperty.EVENT_NAME)
        != FilumEventName.SURVEY_ANSWERED
    ):
        return None

    follow_up_text = event_data.get(FilumTrackEventProperty.FOLLOW_UP_TEXT)
    if not follow_up_text or not re.search(TRANSLATION_FORMAT_REGEX, follow_up_text):
        return None

    survey_id = event_data.get(FilumTrackEventProperty.SURVEY_ID)
    return filum_client.get_survey_default_translations(survey_id)



from filum_utils.enums import BaseEnum


class PublisherType(BaseEnum):
    DATA_CONNECTION = "Data Connection"
    VoC = "Voice of the Customer"
    AUTOMATED_ACTION = "Automated Action"


class RoutePath(BaseEnum):
    CONNECTIONS_DETAIL = "connectionsDetail"
    AUTOMATED_ACTIONS_DETAIL = "automatedActionsDetail"
    CAMPAIGNS_DETAIL = "listeningCampaignDetails"
    ENGAGEMENT_CAMPAIGNS_DETAIL = "engagementCampaignDetails"


class ErrorType:
    INTERNAL = "internal"
    EXTERNAL = "external"


class NotificationErrorMessage:
    INTERNAL = "Internal system error, please contact Filum for support"
    EXTERNAL = "Partner error, please contact Filum for support"


class ToolLogTypeEnum:
    INFO = "info"


class ToolLogCodeEnum:
    EXECUTE = "execute"


class ToolLogActionEnum:
    EXECUTE = "execute"


NOTIFICATION_ERROR_MESSAGE_MAPPINGS = {
    ErrorType.INTERNAL: NotificationErrorMessage.INTERNAL,
    ErrorType.EXTERNAL: NotificationErrorMessage.EXTERNAL,
}



from typing import Any, Dict

from typing_extensions import TypedDict


class PopulateDataParams(TypedDict):
    event: Dict[str, Any]
    user: Dict[str, Any]
    campaign: Dict[str, Any]
    survey_response: Dict[str, Any]
    conversation: Dict[str, Any]
    platform_url: str
    action: Dict[str, Any]



from enum import Enum
from typing import TypedDict


class BaseEnum:
    @classmethod
    def get_list(cls):
        return [getattr(cls, attr) for attr in dir(cls) if attr.isupper()]


class ParentType(BaseEnum):
    AUTOMATED_ACTION = "automated_action"
    CAMPAIGN = "campaign"
    ENGAGEMENT_CAMPAIGN = "engagement_campaign"


class ObjectType(BaseEnum):
    INSTALLED_SOURCE = "installed_source"
    INSTALLED_MINI_APP = "installed_mini_app"
    ACTION = "action"
    TOOL = "tool"


class Organization(TypedDict):
    id: str
    slug: str


class Object(BaseEnum):
    CAMPAIGN = "campaign"
    SEGMENT = "segment"


class BaseStatus(BaseEnum):
    COMPLETED = "completed"


class FileCSVDelimiter(BaseEnum):
    COMMA = ","
    SEMICOLON = ";"
    TAB = "\t"


class Codec(BaseEnum):
    UTF8 = "utf-8"
    UTF16 = "utf-16"
    WINDOW_1258 = "windows-1258"
    ISO_8859_1 = "iso-8859-1"
    CP_1252 = "cp1252"
    VISCII = "viscii"


class EngagementMessageStatus(str, Enum):
    SENT = "sent"
    SENDING_FAILED = "sending_failed"
    SENDING = "sending"


class ActionMetaDataKey(str, Enum):
    PROPERTY_VALUE_MAPPINGS = "property_value_mappings"


class Period(str, Enum):
    MONTHLY = "monthly"



import re
from typing import Optional


class StringFormatter:
    @classmethod
    def remove_accent_vietnamese(cls, s: str) -> str:
        s = re.sub(r"[àáạảãâầấậẩẫăằắặẳẵ]", "a", s)
        s = re.sub(r"[ÀÁẠẢÃĂẰẮẶẲẴÂẦẤẬẨẪ]", "A", s)
        s = re.sub(r"[èéẹẻẽêềếệểễ]", "e", s)
        s = re.sub(r"[ÈÉẸẺẼÊỀẾỆỂỄ]", "E", s)
        s = re.sub(r"[òóọỏõôồốộổỗơờớợởỡ]", "o", s)
        s = re.sub(r"[ÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠ]", "O", s)
        s = re.sub(r"[ìíịỉĩ]", "i", s)
        s = re.sub(r"[ÌÍỊỈĨ]", "I", s)
        s = re.sub(r"[ùúụủũưừứựửữ]", "u", s)
        s = re.sub(r"[ƯỪỨỰỬỮÙÚỤỦŨ]", "U", s)
        s = re.sub(r"[ỳýỵỷỹ]", "y", s)
        s = re.sub(r"[ỲÝỴỶỸ]", "Y", s)
        s = re.sub(r"[đ]", "d", s)
        s = re.sub(r"[Đ]", "D", s)
        return s

    @classmethod
    def transform_phone(cls, phone: Optional[str]) -> Optional[str]:
        if phone == "None":
            phone = None

        if not phone:
            return None

        if phone.startswith("0"):
            return cls._format_phone(phone, "0", True)

        if phone.startswith("+"):
            return cls._format_phone(phone, "+", False)

        return phone

    @classmethod
    def _format_phone(
        cls, phone: str, removed_str: str, is_formatted: bool = False
    ) -> str:
        array_phone = phone.split(removed_str, 1)
        formatted_phone = array_phone[1]
        if is_formatted:
            formatted_phone = "84" + formatted_phone

        return formatted_phone
